/*******************************************************************************
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ, 
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.asitplus.eidas.specific.connector.storage;

import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import at.gv.egiz.eaaf.core.api.storage.ITransactionStorage;
import at.gv.egiz.eaaf.core.exceptions.EAAFException;
import at.gv.egiz.eaaf.core.exceptions.EAAFStorageException;

public class eIDASCacheTransactionStoreDecorator implements ITransactionStorage{
	private static final Logger log = LoggerFactory.getLogger(eIDASCacheTransactionStoreDecorator.class);
	
	@Autowired(required=true) private CacheWitheIDASBackend storage;
	
	@Override
	public void changeKey(String oldKey, String newKey, Object value) throws EAAFException {
		if (containsKey(oldKey)) {
			TransactionStoreElement el = storage.get(oldKey);
			el.setKey(newKey);
			storage.put(newKey, el);
			storage.remove(oldKey);
			
		} else
			throw new EAAFStorageException("No element in TransactionStorage with key: " + oldKey);
		
	}

	@Override
	public List<String> clean(Date now, long dataTimeOut) {
		return storage.clean(now, dataTimeOut);
				
	}

	@Override
	public boolean containsKey(String key) {
		if (key != null) {
			if (storage.get(key) != null)
				return true;
			
		}
			
		return false;
		
	}

	@Override
	public Object get(String key) throws EAAFException {
		if (key != null && containsKey(key)) {
			TransactionStoreElement element = storage.get(key);
			return element.getData();
			
		} else
			return null;	
	}

	@Override
	public <T> T get(String key, Class<T> type) throws EAAFException {
		return get(key, type, -1);
		
	}

	@Override
	public <T> T get(String key, Class<T> type, long dataTimeOut) throws EAAFException {
		if (key != null && containsKey(key)) {
			TransactionStoreElement value = storage.get(key); 
			
			if (dataTimeOut > -1) {
				long now = new Date().getTime();				
				if (now - value.getCreated().getTime() > dataTimeOut) {
					log.info("Transaction-Data with key: " + key + " is out of time.");
					throw new EAAFStorageException("Transaction-Data with key: " + key + " is out of time.");
					
				}	 				 
			}
			
			if (type.isAssignableFrom(value.getData().getClass())) {
				return (T) value.getData();
				
			} else
				log.warn("Can NOT cast '" + value.getClass() + "' to '" + type + "'");
			
		}
					
		return null;
	}

	@Override
	public Object getRaw(String key) throws EAAFException {
		return storage.get(key);
		
	}

	@Override
	public void put(String key, Object value, int dataTimeOut) throws EAAFException {
		TransactionStoreElement element = new TransactionStoreElement();
		element.setKey(key);
		element.setData(value);
		storage.put(key, element);
		
	}

	@Override
	public void putRaw(String key, Object value) throws EAAFException {
		if (value instanceof TransactionStoreElement)
			storage.put(((TransactionStoreElement) value).getKey(), (TransactionStoreElement) value);		
		else 
			log.info(value.getClass().getName() + " is NOT a RAW element of " + ITransactionStorage.class.getName());
					
	}

	@Override
	public void remove(String key) {
		if (containsKey(key)) {
			log.debug("Remove element with key: " + key + " from " + ITransactionStorage.class.getName());
			storage.remove(key);
			
		}
	}

}
